from pritunl.constants import *
from pritunl.organization import Organization
from pritunl.server import Server
import pritunl.utils as utils
from pritunl import app_server
import os
import flask
import tarfile
import time

def tar_add(tar_file, path):
    if os.path.exists(path):
        tar_file.add(path, arcname=os.path.relpath(path, app_server.data_path))

@app_server.app.route('/export', methods=['GET'])
@app_server.app.route('/export/%s.tar' % APP_NAME, methods=['GET'])
@app_server.auth
def export_get():
    data_path = app_server.data_path
    temp_path = os.path.join(data_path, TEMP_DIR)
    empty_temp_path = os.path.join(temp_path, EMPTY_TEMP_DIR)
    data_archive_name = '%s_%s.tar' % (APP_NAME,
        time.strftime('%Y_%m_%d_%H_%M_%S', time.localtime()))
    data_archive_path = os.path.join(temp_path, data_archive_name)

    # Create empty temp directory to recreate temp dirs in tarfile
    if not os.path.exists(empty_temp_path):
        os.makedirs(empty_temp_path)

    tar_file = tarfile.open(data_archive_path, 'w')
    try:
        tar_add(tar_file, os.path.join(data_path, SERVER_CERT_NAME))
        tar_add(tar_file, os.path.join(data_path, SERVER_KEY_NAME))
        tar_add(tar_file, os.path.join(data_path, VERSION_NAME))

        for org in Organization.get_orgs():
            tar_add(tar_file, org.index_path)
            tar_add(tar_file, org.index_attr_path)
            tar_add(tar_file, org.serial_path)
            tar_add(tar_file, org.crl_path)
            tar_add(tar_file, org.get_path())
            print os.path.relpath(os.path.join(org.path, TEMP_DIR),
                    data_path)
            tar_file.add(empty_temp_path,
                arcname=os.path.relpath(os.path.join(org.path, TEMP_DIR),
                    data_path))

            indexed_certs_path = os.path.join(org.path, INDEXED_CERTS_DIR)
            for path in os.listdir(indexed_certs_path):
                tar_add(tar_file, os.path.join(indexed_certs_path, path))

            for user in org.get_users() + [org.ca_cert]:
                tar_add(tar_file, user.reqs_path)
                tar_add(tar_file, user.ssl_conf_path)
                tar_add(tar_file, user.key_path)
                tar_add(tar_file, user.cert_path)
                tar_add(tar_file, user.key_archive_path)
                tar_add(tar_file, user.get_path())

        for server in Server.get_servers():
            tar_add(tar_file, server.dh_param_path)
            tar_add(tar_file, server.ifc_pool_path)
            tar_add(tar_file, server.get_path())
            tar_file.add(empty_temp_path,
                arcname=os.path.relpath(os.path.join(server.path, TEMP_DIR),
                    data_path))
    finally:
        tar_file.close()

    with open(data_archive_path, 'r') as archive_file:
        response = flask.Response(response=archive_file.read(),
            mimetype='application/x-tar')
        response.headers.add('Content-Disposition',
            'inline; filename="%s.tar"' % data_archive_name)

    os.remove(data_archive_path)
    return response



from pritunl.constants import *
from pritunl.organization import Organization
import pritunl.utils as utils
from pritunl import app_server
import os
import flask
import uuid
import time

_key_ids = {}

def _get_key_archive(org_id, user_id):
    org = Organization(org_id)
    user = org.get_user(user_id)
    archive_temp_path = user.build_key_archive()

    with open(archive_temp_path, 'r') as archive_file:
        response = flask.Response(response=archive_file.read(),
            mimetype='application/x-tar')
        response.headers.add('Content-Disposition',
            'inline; filename="%s.tar"' % user.name)

    os.remove(archive_temp_path)
    return response

@app_server.app.route('/key/<org_id>/<user_id>.tar', methods=['GET'])
@app_server.auth
def user_key_archive_get(org_id, user_id):
    return _get_key_archive(org_id, user_id)

@app_server.app.route('/key/<org_id>/<user_id>', methods=['GET'])
@app_server.auth
def user_key_link_get(org_id, user_id):
    key_id = uuid.uuid4().hex
    _key_ids[key_id] = {
        'org_id': org_id,
        'user_id': user_id,
        'timestamp': time.time()
    }
    return utils.jsonify({
        'id': key_id,
        'url': '/key/%s.tar' % key_id
    })

@app_server.app.route('/key/<key_id>.tar', methods=['GET'])
def user_linked_key_archive_get(key_id):
    if key_id not in _key_ids:
        return flask.abort(404)
    elif time.time() - _key_ids[key_id]['timestamp'] > KEY_LINK_TIMEOUT:
        del _key_ids[key_id]
        return flask.abort(404)
    org_id = _key_ids[key_id]['org_id']
    user_id = _key_ids[key_id]['user_id']
    del _key_ids[key_id]
    return _get_key_archive(org_id, user_id)



from constants import *
from event import Event
from database_object import DatabaseObject
import logging
import time
import uuid

logger = logging.getLogger(APP_NAME)

class LogEntry(DatabaseObject):
    column_family = 'log_entries'
    str_columns = ['type', 'message']
    int_columns = ['time']
    cached_columns = ['type', 'message', 'time']
    required_columns = ['type', 'message', 'time']

    def __init__(self, id=None, type=None, message=None):
        DatabaseObject.__init__(self)

        if id is None:
            self.id = uuid.uuid4().hex
            self.type = type or INFO
            self.time = int(time.time())
            self.message = message
            Event(type=LOG_UPDATED)
        else:
            self.id = id

    @staticmethod
    def get_log_entries():
        logs = []
        logs_dict = {}
        logs_sort = []

        logger.debug('Getting log entries.')

        logs_query = LogEntry.db.get(LogEntry.column_family)
        for log_id in logs_query:
            log = logs_query[log_id]

            if not DatabaseObject.validate(LogEntry, log_id, log):
                continue

            log['time'] = int(log['time'])

            time_id = '%s_%s' % (log['time'], log_id)
            logs_dict[time_id] = LogEntry(id=log_id)
            logs_sort.append(time_id)

        for time_id in reversed(sorted(logs_sort)):
            logs.append(logs_dict[time_id])

        for log in logs[DEFAULT_LOG_LIMIT:]:
            logger.debug('Pruning log entry from database. %r' % {
                'log_id': log.id,
            })
            LogEntry.db.remove(LogEntry.column_family, log.id)

        return logs[:DEFAULT_LOG_LIMIT]



from pritunl.constants import *
from pritunl.log_entry import LogEntry
import pritunl.utils as utils
from pritunl import app_server

@app_server.app.route('/log', methods=['GET'])
@app_server.auth
def log_get():
    log_entries = []

    for log_entry in LogEntry.get_log_entries():
        log_entries.append({
            'id': log_entry.id,
            'time': log_entry.time,
            'message': log_entry.message,
        })

    return utils.jsonify(log_entries)



from pritunl.constants import *
import pritunl.utils as utils
from pritunl import app_server
import flask

@app_server.app.route('/password', methods=['POST'])
@app_server.auth
def password_post():
    password = flask.request.json['password'][:512]
    app_server.set_password(password)
    return utils.jsonify({})



from pritunl.constants import *
from pritunl.organization import Organization
import pritunl.utils as utils
from pritunl import app_server
import flask

@app_server.app.route('/user/<org_id>', methods=['GET'])
@app_server.auth
def user_get(org_id):
    org = Organization(org_id)
    otp_auth = False
    users = []
    users_dict = {}
    users_sort = []
    clients = {}

    for server in org.get_servers():
        if server.otp_auth:
            otp_auth = True
        server_clients = server.get_clients()
        for client_id in server_clients:
            client = server_clients[client_id]
            if client_id not in clients:
                clients[client_id] = {}
            clients[client_id][server.id] = client

    for user in org.get_users():
        name_id = '%s_%s' % (user.name, user.id)
        users_sort.append(name_id)
        users_dict[name_id] = {
            'id': user.id,
            'organization': org.id,
            'name': user.name,
            'type': user.type,
            'status': True if user.id in clients else False,
            'otp_auth': otp_auth,
            'otp_secret': user.otp_secret,
            'servers': clients[user.id] if user.id in clients else {},
        }

    for name_id in sorted(users_sort):
        users.append(users_dict[name_id])

    return utils.jsonify(users)

@app_server.app.route('/user/<org_id>', methods=['POST'])
@app_server.auth
def user_post(org_id):
    org = Organization(org_id)
    name = flask.request.json['name']
    name = ''.join(x for x in name if x.isalnum() or x in NAME_SAFE_CHARS)
    user = org.new_user(CERT_CLIENT, name)

    return utils.jsonify({})

@app_server.app.route('/user/<org_id>/<user_id>', methods=['PUT'])
@app_server.auth
def user_put(org_id, user_id):
    org = Organization(org_id)
    user = org.get_user(user_id)
    name = flask.request.json['name']
    name = ''.join(x for x in name if x.isalnum() or x in NAME_SAFE_CHARS)
    user.rename(name)
    return utils.jsonify({})

@app_server.app.route('/user/<org_id>/<user_id>', methods=['DELETE'])
@app_server.auth
def user_delete(org_id, user_id):
    org = Organization(org_id)
    user = org.get_user(user_id)
    user_id = user.id
    user.remove()
    for server in org.get_servers():
        server_clients = server.get_clients()
        if user_id in server_clients:
            server.restart()
    return utils.jsonify({})

@app_server.app.route('/user/<org_id>/<user_id>/otp_secret',
    methods=['DELETE'])
@app_server.auth
def user_otp_secret_delete(org_id, user_id):
    org = Organization(org_id)
    user = org.get_user(user_id)
    user.generate_otp_secret()
    return utils.jsonify({})



import os
import re

TEMPLATES_DIR = '../www/templates'

for html_name in os.listdir(TEMPLATES_DIR):
    html_path = os.path.join(TEMPLATES_DIR, html_name)
    with open(html_path, 'r') as html_file:
        print re.sub('<%(.*?)%>', '', html_file.read(), flags=re.S)



from pritunl.constants import *
from pritunl.organization import Organization
import pritunl.utils as utils
from pritunl import app_server
import flask

@app_server.app.route('/organization', methods=['GET'])
@app_server.auth
def org_get():
    orgs = []
    orgs_dict = {}
    orgs_sort = []

    for org in Organization.get_orgs():
        name_id = '%s_%s' % (org.name, org.id)
        orgs_sort.append(name_id)
        orgs_dict[name_id] = {
            'id': org.id,
            'name': org.name,
        }

    for name_id in sorted(orgs_sort):
        orgs.append(orgs_dict[name_id])

    return utils.jsonify(orgs)

@app_server.app.route('/organization', methods=['POST'])
@app_server.auth
def org_post():
    name = flask.request.json['name']
    name = ''.join(x for x in name if x.isalnum() or x in NAME_SAFE_CHARS)
    org = Organization(name=name)
    return utils.jsonify({})

@app_server.app.route('/organization/<org_id>', methods=['PUT'])
@app_server.auth
def org_put(org_id):
    org = Organization(org_id)
    name = flask.request.json['name']
    name = ''.join(x for x in name if x.isalnum() or x in NAME_SAFE_CHARS)
    org.rename(name)
    return utils.jsonify({})

@app_server.app.route('/organization/<org_id>', methods=['DELETE'])
@app_server.auth
def org_delete(org_id):
    org = Organization(org_id)
    org.remove()
    return utils.jsonify({})



from pritunl.constants import *
from pritunl.server import Server
from pritunl.organization import Organization
from event import Event
import pritunl.utils as utils
from pritunl import app_server
import flask

def _network_not_valid():
    return utils.jsonify({
        'error': NETWORK_NOT_VALID,
        'error_msg': NETWORK_NOT_VALID_MSG,
    }, 400)

def _interface_not_valid():
        return utils.jsonify({
            'error': INTERFACE_NOT_VALID,
            'error_msg': INTERFACE_NOT_VALID_MSG,
        }, 400)

def _port_not_valid():
        return utils.jsonify({
            'error': PORT_NOT_VALID,
            'error_msg': PORT_NOT_VALID_MSG,
        }, 400)

def _local_network_not_valid():
    return utils.jsonify({
        'error': LOCAL_NETWORK_NOT_VALID,
        'error_msg': LOCAL_NETWORK_NOT_VALID_MSG,
    }, 400)

@app_server.app.route('/server', methods=['GET'])
@app_server.auth
def server_get():
    servers = []
    servers_dict = {}
    servers_sort = []

    for server in Server.get_servers():
        server_orgs = server.get_orgs()
        users_count = 0
        for org in server_orgs:
            for user in org.get_users():
                if user.type != CERT_CLIENT:
                    continue
                users_count += 1

        name_id = '%s_%s' % (server.name, server.id)
        servers_sort.append(name_id)
        servers_dict[name_id] = {
            'id': server.id,
            'name': server.name,
            'status': 'online' if server.status else 'offline',
            'uptime': server.uptime,
            'users_online': len(server.get_clients()),
            'users_total': users_count,
            'network': server.network,
            'interface': server.interface,
            'port': server.port,
            'protocol': server.protocol,
            'local_network': server.local_network,
            'public_address': server.public_address,
            'otp_auth': True if server.otp_auth else False,
            'lzo_compression': server.lzo_compression,
            'debug': True if server.debug else False,
            'org_count': len(server_orgs),
        }

    for name_id in sorted(servers_sort):
        servers.append(servers_dict[name_id])

    return utils.jsonify(servers)

@app_server.app.route('/server', methods=['POST'])
@app_server.app.route('/server/<server_id>', methods=['PUT'])
@app_server.auth
def server_put_post(server_id=None):
    name = flask.request.json['name']
    name = ''.join(x for x in name if x.isalnum() or x in NAME_SAFE_CHARS)
    network = flask.request.json['network']
    interface = flask.request.json['interface']
    port = flask.request.json['port']
    protocol = flask.request.json['protocol'].lower()
    local_network = flask.request.json['local_network']
    if local_network:
        local_network = local_network
    public_address = flask.request.json['public_address']
    public_address = ''.join(
        x for x in public_address if x.isalnum() or x == '.')
    debug = True if flask.request.json['debug'] else False
    otp_auth = True if flask.request.json['otp_auth'] else False
    lzo_compression = True if flask.request.json['lzo_compression'] else False

    # Network
    network_split = network.split('/')
    if len(network_split) != 2:
        return _network_not_valid()

    address = network_split[0].split('.')
    if len(address) != 4:
        return _network_not_valid()
    for i, value in enumerate(address):
        try:
            address[i] = int(value)
        except ValueError:
            return _network_not_valid()
    if address[0] != 10:
        return _network_not_valid()

    if address[1] > 255 or address[1] < 0 or \
            address[2] > 255 or address[2] < 0:
        return _network_not_valid()

    if address[3] != 0:
        return _network_not_valid()

    try:
        subnet = int(network_split[1])
    except ValueError:
        return _network_not_valid()

    if subnet < 8 or subnet > 24:
        return _network_not_valid()

    # Interface
    if interface[:3] != 'tun':
        return _interface_not_valid()

    try:
        interface_num = int(interface[3:])
    except ValueError:
        return _interface_not_valid()

    if interface_num > 64:
        return _interface_not_valid()

    interface = interface[:3] + str(interface_num)

    # Port
    try:
        port = int(port)
    except ValueError:
        return _port_not_valid()

    if port < 1 or port > 65535:
        return _port_not_valid()

    # Protocol
    if protocol not in ['udp', 'tcp']:
        return utils.jsonify({
            'error': PROTOCOL_NOT_VALID,
            'error_msg': PROTOCOL_NOT_VALID_MSG,
        }, 400)

    # Local network
    if local_network:
        local_network_split = local_network.split('/')
        if len(local_network_split) != 2:
            return _local_network_not_valid()

        address = local_network_split[0].split('.')
        if len(address) != 4:
            return _local_network_not_valid()
        for i, value in enumerate(address):
            try:
                address[i] = int(value)
            except ValueError:
                return _local_network_not_valid()
        if address[0] > 255 or address[0] < 0 or \
                address[1] > 255 or address[1] < 0 or \
                address[2] > 255 or address[2] < 0 or \
                address[3] > 254 or address[3] < 0:
            return _local_network_not_valid()

        try:
            subnet = int(local_network_split[1])
        except ValueError:
            return _local_network_not_valid()

        if subnet < 8 or subnet > 30:
            return _local_network_not_valid()

    for server in Server.get_servers():
        if server.id == server_id:
            continue
        elif server.network == network:
            return utils.jsonify({
                'error': NETWORK_IN_USE,
                'error_msg': NETWORK_IN_USE_MSG,
            }, 400)
        elif server.interface == interface:
            return utils.jsonify({
                'error': INTERFACE_IN_USE,
                'error_msg': INTERFACE_IN_USE_MSG,
            }, 400)
        elif server.port == port and server.protocol == protocol:
            return utils.jsonify({
                'error': PORT_PROTOCOL_IN_USE,
                'error_msg': PORT_PROTOCOL_IN_USE_MSG,
            }, 400)

    if not server_id:
        server = Server(
            name=name,
            network=network,
            interface=interface,
            port=port,
            protocol=protocol,
            local_network=local_network,
            public_address=public_address,
            otp_auth=otp_auth,
            lzo_compression=lzo_compression,
            debug=debug,
        )
    else:
        server = Server(id=server_id)
        if server.status:
            return utils.jsonify({
                'error': SERVER_NOT_OFFLINE,
                'error_msg': SERVER_NOT_OFFLINE_SETTINGS_MSG,
            }, 400)
        server.name = name
        server.network = network
        server.interface = interface
        server.port = port
        server.protocol = protocol
        server.local_network = local_network
        server.public_address = public_address
        server.otp_auth = otp_auth
        server.lzo_compression = lzo_compression
        server.debug = debug
        server.commit()

    Event(type=USERS_UPDATED)

    return utils.jsonify({})

@app_server.app.route('/server/<server_id>', methods=['DELETE'])
@app_server.auth
def server_delete(server_id):
    server = Server(server_id)
    server.remove()
    return utils.jsonify({})

@app_server.app.route('/server/<server_id>/organization', methods=['GET'])
@app_server.auth
def server_org_get(server_id):
    orgs = []
    orgs_dict = {}
    orgs_sort = []
    server = Server(server_id)

    for org in server.get_orgs():
        name_id = '%s_%s' % (org.name, org.id)
        orgs_sort.append(name_id)
        orgs_dict[name_id] = {
            'id': org.id,
            'server': server.id,
            'name': org.name,
        }

    for name_id in sorted(orgs_sort):
        orgs.append(orgs_dict[name_id])

    return utils.jsonify(orgs)

@app_server.app.route('/server/<server_id>/organization/<org_id>',
    methods=['PUT'])
@app_server.auth
def server_org_put(server_id, org_id):
    server = Server(server_id)
    if server.status:
        return utils.jsonify({
            'error': SERVER_NOT_OFFLINE,
            'error_msg': SERVER_NOT_OFFLINE_ATTACH_ORG_MSG,
        }, 400)
    server.add_org(org_id)
    return utils.jsonify({})

@app_server.app.route('/server/<server_id>/organization/<org_id>',
    methods=['DELETE'])
@app_server.auth
def server_org_delete(server_id, org_id):
    server = Server(server_id)
    if server.status:
        return utils.jsonify({
            'error': SERVER_NOT_OFFLINE,
            'error_msg': SERVER_NOT_OFFLINE_DETACH_ORG_MSG,
        }, 400)
    server.remove_org(org_id)
    return utils.jsonify({})

@app_server.app.route('/server/<server_id>/<operation>', methods=['PUT'])
@app_server.auth
def server_operation_put(server_id, operation):
    server = Server(server_id)
    if operation == START:
        server.start()
    if operation == STOP:
        server.stop()
    elif operation == RESTART:
        server.restart()
    return utils.jsonify({})

@app_server.app.route('/server/<server_id>/output', methods=['GET'])
@app_server.auth
def server_output_get(server_id):
    server = Server(server_id)
    return utils.jsonify({
        'id': server.id,
        'output': server.get_output(),
    })

@app_server.app.route('/server/<server_id>/output', methods=['DELETE'])
@app_server.auth
def server_output_delete(server_id):
    server = Server(server_id)
    server.clear_output()
    return utils.jsonify({})



from constants import *
from pritunl import app_server, openssl_lock
from config import Config
from log_entry import LogEntry
from event import Event
import uuid
import tarfile
import os
import subprocess
import logging
import hashlib
import base64

logger = logging.getLogger(APP_NAME)

class User(Config):
    str_options = ['name', 'otp_secret']

    def __init__(self, org, id=None, name=None, type=None):
        Config.__init__(self)
        self.org = org
        self.id = id

        if type is not None:
            self.type = type

        if self.id is None:
            if type == CERT_CA:
                self.id = CA_CERT_ID
            elif type is None:
                raise AttributeError('Type must be specified')
            else:
                self.id = uuid.uuid4().hex
            self._initialized = False
        else:
            self._initialized = True

        self.reqs_path = os.path.join(self.org.path, REQS_DIR,
            '%s.csr' % self.id)
        self.ssl_conf_path = os.path.join(self.org.path, TEMP_DIR,
            '%s.conf' % self.id)
        self.key_path = os.path.join(self.org.path, KEYS_DIR,
            '%s.key' % self.id)
        self.cert_path = os.path.join(self.org.path, CERTS_DIR,
            '%s.crt' % self.id)
        self.key_archive_path = os.path.join(self.org.path,
            TEMP_DIR, '%s.tar' % self.id)
        self.set_path(os.path.join(self.org.path, USERS_DIR,
            '%s.conf' % self.id))

        if name is not None:
            self.name = name

        if not self._initialized:
            self._initialize()

    def __getattr__(self, name):
        if name == 'type':
            self.type = self._load_type()
            return self.type
        return Config.__getattr__(self, name)

    def _upgrade_0_10_2(self):
        if not self.otp_secret:
            logger.debug('Upgrading user from v0.10.1... %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            self._generate_otp_secret()
            self.commit(0600)

    def _initialize(self):
        self._create_ssl_conf()
        self._cert_request()
        self._generate_otp_secret()
        self.commit(0600)
        self._cert_create()
        self._delete_ssl_conf()
        LogEntry(message='Created new user.')
        Event(type=USERS_UPDATED)

    def _cert_request(self):
        openssl_lock.acquire()
        try:
            args = [
                'openssl', 'req', '-new', '-batch',
                '-config', self.ssl_conf_path,
                '-out', self.reqs_path,
                '-keyout', self.key_path,
                '-reqexts', '%s_req_ext' % self.type,
            ]
            subprocess.check_call(args, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            logger.exception('Failed to create user cert requests. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            raise
        finally:
            openssl_lock.release()
        os.chmod(self.key_path, 0600)

    def _cert_create(self):
        openssl_lock.acquire()
        try:
            args = ['openssl', 'ca', '-batch']
            if self.type == CERT_CA:
                args += ['-selfsign']
            args += [
                '-config', self.ssl_conf_path,
                '-in', self.reqs_path,
                '-out', self.cert_path,
                '-extensions', '%s_ext' % self.type,
            ]
            subprocess.check_call(args, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            logger.exception('Failed to create user cert. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            raise
        finally:
            openssl_lock.release()

    def _create_ssl_conf(self):
        conf_data = CERT_CONF % (self.org.id, self.org.path,
            app_server.key_bits, self.id)
        with open(self.ssl_conf_path, 'w') as conf_file:
            conf_file.write(conf_data)

    def _delete_ssl_conf(self):
        os.remove(self.ssl_conf_path)

    def _generate_otp_secret(self):
        sha_hash = hashlib.sha512()
        sha_hash.update(os.urandom(8192))
        byte_hash = sha_hash.digest()
        for i in xrange(6):
            sha_hash = hashlib.sha512()
            sha_hash.update(byte_hash)
            byte_hash = sha_hash.digest()
        self.otp_secret = base64.b32encode(byte_hash)[:DEFAULT_OTP_SECRET_LEN]

    def generate_otp_secret(self):
        self._generate_otp_secret()
        self.commit(0600)
        Event(type=USERS_UPDATED)

    def _load_type(self):
        with open(self.cert_path, 'r') as cert_file:
            cert_data = cert_file.read()
            if 'CA:TRUE' in cert_data:
                return CERT_CA
            elif 'TLS Web Server Authentication' in cert_data:
                return CERT_SERVER
            else:
                return  CERT_CLIENT

    def _revoke(self, reason):
        if self.id == CA_CERT_ID:
            raise TypeError('Cannot revoke ca cert')

        if not os.path.isfile(self.cert_path):
            logger.warning('Skipping revoke of non existent user. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            return

        openssl_lock.acquire()
        try:
            self._create_ssl_conf()
            args = ['openssl', 'ca', '-batch',
                '-config', self.ssl_conf_path,
                '-revoke', self.cert_path,
                '-crl_reason', reason
            ]
            proc = subprocess.Popen(args, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            returncode = proc.wait()
            if returncode != 0:
                err_output = proc.communicate()[1]
                if 'ERROR:Already revoked' not in err_output:
                    raise subprocess.CalledProcessError(returncode, args)
            self._delete_ssl_conf()
        except subprocess.CalledProcessError:
            logger.exception('Failed to revoke user cert. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            raise
        finally:
            openssl_lock.release()
        self.org.generate_crl()

    def _build_key_archive(self):
        user_key_arcname = '%s_%s.key' % (self.org.name, self.name)
        user_cert_arcname = '%s_%s.crt' % (self.org.name, self.name)

        tar_file = tarfile.open(self.key_archive_path, 'w')
        try:
            tar_file.add(self.key_path, arcname=user_key_arcname)
            tar_file.add(self.cert_path, arcname=user_cert_arcname)
            for server in self.org.get_servers():
                server_cert_arcname = '%s_%s_%s.crt' % (
                    self.org.name, self.name, server.name)
                server_conf_path = os.path.join(self.org.path,
                    TEMP_DIR, '%s_%s.ovpn' % (self.id, server.id))
                server_conf_arcname = '%s_%s_%s.ovpn' % (
                    self.org.name, self.name, server.name)
                server.generate_ca_cert()
                tar_file.add(server.ca_cert_path, arcname=server_cert_arcname)

                client_conf = OVPN_CLIENT_CONF % (
                    server.protocol,
                    server.public_address, server.port,
                    server_cert_arcname,
                    user_cert_arcname,
                    user_key_arcname,
                )

                if server.otp_auth:
                    client_conf += 'auth-user-pass\n'

                with open(server_conf_path, 'w') as ovpn_conf:
                    ovpn_conf.write(client_conf)
                tar_file.add(server_conf_path, arcname=server_conf_arcname)
                os.remove(server_conf_path)
        finally:
            tar_file.close()

        return self.key_archive_path

    def _get_cert_block(self, cert_path):
        with open(cert_path) as cert_file:
            cert_file = cert_file.read()
            start_index = cert_file.index('-----BEGIN CERTIFICATE-----')
            end_index = cert_file.index('-----END CERTIFICATE-----') + 25
            return cert_file[start_index:end_index]

    def _build_inline_key_archive(self):
        tar_file = tarfile.open(self.key_archive_path, 'w')
        try:
            for server in self.org.get_servers():
                server_conf_path = os.path.join(self.org.path,
                    TEMP_DIR, '%s_%s.ovpn' % (self.id, server.id))
                server_conf_arcname = '%s_%s_%s.ovpn' % (
                    self.org.name, self.name, server.name)
                server.generate_ca_cert()

                client_conf = OVPN_CLIENT_CONF % (
                    server.protocol,
                    server.public_address, server.port,
                    '[inline]',
                    '[inline]',
                    '[inline]',
                )

                if server.otp_auth:
                    client_conf += 'auth-user-pass\n'

                client_conf += '<ca>\n%s\n</ca>\n' % self._get_cert_block(
                    server.ca_cert_path)
                client_conf += '<cert>\n%s\n</cert>\n' % self._get_cert_block(
                    self.cert_path)
                client_conf += '<key>\n%s\n</key>\n' % open(
                    self.key_path).read().strip()

                with open(server_conf_path, 'w') as ovpn_conf:
                    os.chmod(server_conf_path, 0600)
                    ovpn_conf.write(client_conf)
                tar_file.add(server_conf_path, arcname=server_conf_arcname)
                os.remove(server_conf_path)
        finally:
            tar_file.close()

        return self.key_archive_path

    def build_key_archive(self):
        if app_server.inline_certs:
            return self._build_inline_key_archive()
        else:
            return self._build_key_archive()

    def rename(self, name):
        self.name = name
        self.commit()
        Event(type=USERS_UPDATED)

    def remove(self, reason=UNSPECIFIED):
        self._revoke(reason)

        try:
            os.remove(self.reqs_path)
        except OSError, error:
            logger.debug('Failed to remove user reqs file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        try:
            os.remove(self.ssl_conf_path)
        except OSError, error:
            pass

        try:
            os.remove(self.key_path)
        except OSError, error:
            logger.debug('Failed to remove user key file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        try:
            os.remove(self.cert_path)
        except OSError, error:
            logger.debug('Failed to remove user cert file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        try:
            os.remove(self.get_path())
        except OSError, error:
            logger.debug('Failed to remove user conf file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        Event(type=USERS_UPDATED)
        LogEntry(message='Deleted user.')



from pritunl.constants import *
import pritunl.utils as utils
from pritunl.organization import Organization
from pritunl.server import Server
from pritunl import app_server, __version__

@app_server.app.route('/status', methods=['GET'])
@app_server.auth
def status_get():
    orgs = Organization.get_orgs()
    orgs_count = len(orgs)

    servers_count = 0
    servers_online_count = 0
    clients_count = 0
    for server in Server.get_servers():
        servers_count += 1
        if server.status:
            servers_online_count += 1
        clients_count += len(server.get_clients())

    users_count = 0
    for org in orgs:
        for user in org.get_users():
            if user.type != CERT_CLIENT:
                continue
            users_count += 1

    if not app_server.public_ip:
        app_server.load_public_ip()

    return utils.jsonify({
        'orgs_available': orgs_count,
        'orgs_total': orgs_count,
        'users_online': clients_count,
        'users_total': users_count,
        'servers_online': servers_online_count,
        'servers_total': servers_count,
        'server_version': __version__,
        'public_ip': app_server.public_ip,
    })



APP_NAME = 'pritunl'
APP_NAME_FORMATED = 'Pritunl'
CONF_FILENAME = '%s.conf' % APP_NAME

PUBLIC_IP_SERVER = 'http://ip.pritunl.com/'

SAVED = 'saved'
UNSAVED = 'unsaved'

START = 'start'
STOP = 'stop'
RESTART = 'restart'

NAME_SAFE_CHARS = ['-', '_', '@', '.']

KEY_LINK_TIMEOUT = 3600
DEFAULT_PRIMARY_INTERFACE = 'eth0'
DEFAULT_SESSION_TIMEOUT = 86400
DEFAULT_PASSWORD = 'admin'
PASSWORD_SALT = '2511cebca93d028393735637bbc8029207731fcf'
DEFAULT_DB_PATH = '/var/lib/pritunl/pritunl.db'
DEFAULT_WWW_PATH = '/usr/share/pritunl/www'
DEFAULT_DATA_PATH = '/var/lib/pritunl'
DEFAULT_LOG_LIMIT = 20
DEFAULT_KEY_BITS = 4096
DEFAULT_DH_PARAM_BITS = 1536
DEFAULT_OTP_SECRET_LEN = 16

INFO = 'info'
WARNING = 'warning'
ERROR = 'error'

VERSION_NAME = 'version'
ORGS_DIR = 'organizations'
SERVERS_DIR = 'servers'
REQS_DIR = 'reqs'
KEYS_DIR = 'keys'
CERTS_DIR = 'certs'
USERS_DIR = 'users'
INDEXED_CERTS_DIR = 'indexed_certs'
TEMP_DIR = 'temp'
EMPTY_TEMP_DIR = 'empty_temp'
INDEX_NAME = 'index'
SERIAL_NAME = 'serial'
CRL_NAME = 'ca.crl'
TLS_VERIFY_NAME = 'tls_verify.py'
USER_PASS_VERIFY_NAME = 'user_pass_verify.py'
OVPN_CONF_NAME = 'openvpn.conf'
OVPN_STATUS_NAME = 'status'
OVPN_CA_NAME = 'ca.crt'
IFC_POOL_NAME = 'ifc_pool'
DH_PARAM_NAME = 'dh_param.pem'
SERVER_USER_PREFIX = 'server_'
SERVER_CERT_NAME = 'server.crt'
SERVER_KEY_NAME = 'server.key'
OTP_JSON_NAME = 'otp.json'
AUTH_LOG_NAME = 'auth.log'

CA_CERT_ID = 'ca'
CERT_CA = 'ca'
CERT_SERVER = 'server'
CERT_CLIENT = 'client'

UNSPECIFIED = 'unspecified'
KEY_COMPROMISE = 'keyCompromise'
CA_COMPROMISE = 'CACompromise'
AFFILIATION_CHANGED = 'affiliationChanged'
SUPERSEDED = 'superseded'
CESSATION_OF_OPERATION = 'cessationOfOperation'
CERTIFICATE_HOLD = 'certificateHold'
REMOVE_FROM_CRL = 'removeFromCRL'

ORGS_UPDATED = 'organizations_updated'
USERS_UPDATED = 'users_updated'
LOG_UPDATED = 'log_updated'
SERVERS_UPDATED = 'servers_updated'
SERVER_ORGS_UPDATED = 'server_organizations_updated'
SERVER_OUTPUT_UPDATED = 'server_output_updated'

CERT_CONF = """[ default ]
ca = %s
dir = %s

[ req ]
default_bits = %s
default_md = sha1
encrypt_key = no
utf8 = yes
string_mask = utf8only
prompt = no
distinguished_name = req_dn

[ req_dn ]
organizationName = $ca
commonName = %s

[ ca_req_ext ]
keyUsage = critical,keyCertSign,cRLSign
basicConstraints = critical,CA:true
subjectKeyIdentifier = hash

[ server_req_ext ]
keyUsage = critical,digitalSignature,keyEncipherment
extendedKeyUsage = serverAuth,clientAuth
subjectKeyIdentifier = hash

[ client_req_ext ]
keyUsage = critical,digitalSignature,keyEncipherment
extendedKeyUsage = clientAuth
subjectKeyIdentifier = hash

[ ca ]
default_ca = root_ca

[ root_ca ]
database = $dir/index
serial = $dir/serial
new_certs_dir = $dir/indexed_certs
certificate = $dir/certs/ca.crt
private_key = $dir/keys/ca.key
default_days = 3652
default_crl_days = 365
default_md = sha1
policy = ca_policy
crl_extensions = crl_ext

[ ca_policy ]
organizationName = match
commonName = supplied

[ ca_ext ]
keyUsage = critical,keyCertSign,cRLSign
basicConstraints = critical,CA:true
subjectKeyIdentifier = hash
authorityKeyIdentifier = keyid:always

[ crl_ext ]
authorityKeyIdentifier = keyid:always

[ server_ext ]
keyUsage = critical,digitalSignature,keyEncipherment
basicConstraints = CA:false
extendedKeyUsage = serverAuth,clientAuth
subjectKeyIdentifier = hash
authorityKeyIdentifier = keyid:always

[ client_ext ]
keyUsage = critical,digitalSignature,keyEncipherment
basicConstraints = CA:false
extendedKeyUsage = clientAuth
subjectKeyIdentifier = hash
authorityKeyIdentifier = keyid:always
"""

AUTH_NOT_VALID = 'auth_not_valid'
AUTH_NOT_VALID_MSG = 'Username or password is not valid.'

NETWORK_NOT_VALID = 'network_not_valid'
NETWORK_NOT_VALID_MSG = 'Network address is not valid, format must be ' + \
    '"10.[0-255].[0-255].0/[8-24]" such as "10.12.32.0/24".'

LOCAL_NETWORK_NOT_VALID = 'local_network_not_valid'
LOCAL_NETWORK_NOT_VALID_MSG = 'Local network address is not valid, ' + \
    'format must be "[0-255].[0-255].[0-255].[0-254]/[8-30]" such as ' + \
    '"10.0.0.0/8".'

PORT_NOT_VALID = 'port_not_valid'
PORT_NOT_VALID_MSG = 'Port number is not valid, must be between 1 and 65535.'

INTERFACE_NOT_VALID = 'interface_not_valid'
INTERFACE_NOT_VALID_MSG = 'Interface is not valid, must be ' + \
    '"tun[0-64]" example "tun0".'

PROTOCOL_NOT_VALID = 'protocol_not_valid'
PROTOCOL_NOT_VALID_MSG = 'Protocol is not valid, must be "udp" or "tcp".'

NETWORK_IN_USE = 'network_in_use'
NETWORK_IN_USE_MSG = 'Network address is already in use.'

INTERFACE_IN_USE = 'interface_in_use'
INTERFACE_IN_USE_MSG = 'Tunnel interface is already in use.'

PORT_PROTOCOL_IN_USE = 'port_protocol_in_use'
PORT_PROTOCOL_IN_USE_MSG = 'Port and protocol is already in use.'

SERVER_NOT_OFFLINE = 'server_not_offline'
SERVER_NOT_OFFLINE_SETTINGS_MSG = 'Server must be offline to modify settings.'
SERVER_NOT_OFFLINE_ATTACH_ORG_MSG = 'Server must be offline to attach ' + \
    'an organization.'
SERVER_NOT_OFFLINE_DETACH_ORG_MSG = 'Server must be offline to detach ' + \
    'an organization.'

OVPN_SERVER_CONF = """port %s
proto %s
dev %s
ca %s
cert %s
key %s
tls-verify %s
dh %s
server %s
ifconfig-pool-persist %s
push "%s"
keepalive 4 10
persist-tun
status %s 1
status-version 2
script-security 2
verb %s
mute %s
"""

OVPN_CLIENT_CONF = """client
dev tun
proto %s
remote %s %s
nobind
persist-tun
ca %s
cert %s
key %s
verb 2
mute 3
"""

# Script will run in python 2 and 3
TLS_VERIFY_SCRIPT = """#!/usr/bin/env python
import os
import sys
VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
DATA_PATH = '%s'
ORGS_DIR = '%s'
AUTH_LOG_NAME = '%s'
INDEX_NAME = '%s'
auth_log_path = os.path.join(DATA_PATH, AUTH_LOG_NAME)
orgs_path = os.path.join(DATA_PATH, ORGS_DIR)

def log_write(line):
    with open(auth_log_path, 'a') as auth_log_file:
        auth_log_file.write('[TIME=%%s]%%s\\n' %% (int(time.time()), line))

# Get org and common_name from argv
arg = sys.argv[2]
arg = ''.join(x for x in arg if x in VALID_CHARS)
o_index = arg.find('O=')
cn_index = arg.find('CN=')
if o_index < 0 or cn_index < 0:
    log_write('[FAILED] Missing organization or user id from args')
    raise AttributeError('Missing organization or user id from args')
if o_index > cn_index:
    org = arg[o_index + 2:]
    common_name = arg[3:o_index]
else:
    org = arg[2:cn_index]
    common_name = arg[cn_index + 3:]
if not org or not common_name:
    log_write('[FAILED] Missing organization or user id from args')
    raise AttributeError('Missing organization or user id from args')

# Check that common_name is valid
with open(os.path.join(orgs_path, org, INDEX_NAME), 'r') as index_file:
    for line in index_file.readlines():
        if 'O=%%s' %% org in line and 'CN=%%s' %% common_name in line:
            if line[0] == 'V':
                exit(0)
            log_write('[ORG=%%s][UID=%%s][FAILED] User id is not valid' %% (
                org, common_name))
            raise AttributeError('User id is not valid')
log_write('[ORG=%%s][UID=%%s][FAILED] User id not found' %% (
    org, common_name))
raise LookupError('Common name not found')
"""

# Script will run in python 2 and 3
USER_PASS_VERIFY_SCRIPT = """#!/usr/bin/env python
import os
import sys
import time
import struct
import hmac
import hashlib
import base64
import json
VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
DATA_PATH = '%s'
ORGS_DIR = '%s'
USERS_DIR = '%s'
TEMP_DIR = '%s'
AUTH_LOG_NAME = '%s'
OTP_JSON_NAME = '%s'
auth_log_path = os.path.join(DATA_PATH, AUTH_LOG_NAME)
orgs_path = os.path.join(DATA_PATH, ORGS_DIR)

def log_write(line):
    with open(auth_log_path, 'a') as auth_log_file:
        auth_log_file.write('[TIME=%%s]%%s\\n' %% (int(time.time()), line))

# Get org and common_name from environ
tls_env = os.environ.get('tls_id_0')
if not tls_env:
    log_write('[FAILED] Missing organization or user id from environ')
    raise AttributeError('Missing organization or user id from environ')
tls_env = ''.join(x for x in tls_env if x in VALID_CHARS)
o_index = tls_env.find('O=')
cn_index = tls_env.find('CN=')
if o_index < 0 or cn_index < 0:
    log_write('[FAILED] Missing organization or user id from environ')
    raise AttributeError('Missing organization or user id from environ')
if o_index > cn_index:
    org = tls_env[o_index + 2:]
    common_name = tls_env[3:o_index]
else:
    org = tls_env[2:cn_index]
    common_name = tls_env[cn_index + 3:]
if not org or not common_name:
    log_write('[FAILED] Missing organization or user id from environ')
    raise AttributeError('Missing organization or user id from environ')

# Get username and password from input file
with open(sys.argv[1], 'r') as auth_file:
    username, password = [x.strip() for x in auth_file.readlines()[:2]]
password = password[:6]
if not password.isdigit():
    log_write('[ORG=%%s][UID=%%s][FAILED] Authenticator code is invalid' %% (
        org, common_name))
    raise TypeError('Authenticator code is invalid')

# Get secretkey from user conf
secretkey = None
with open(os.path.join(orgs_path, org, USERS_DIR,
        '%%s.conf' %% common_name), 'r') as user_conf_file:
    for line in user_conf_file.readlines():
        if 'otp_secret=' in line:
            secretkey = line.strip().replace('otp_secret=', '')
            break
if not secretkey:
    log_write('[ORG=%%s][UID=%%s][FAILED] Missing otp_secret in user conf' %% (
        org, common_name))
    raise AttributeError('Missing otp_secret in user conf')

# Check password with secretkey
padding = 8 - len(secretkey) %% 8
if padding != 8:
    secretkey = secretkey.ljust(len(secretkey) + padding, '=')
secretkey = base64.b32decode(secretkey.upper())
valid_codes = []
epoch = int(time.time() / 30)
for epoch_offset in range(-1, 2):
    value = struct.pack('>q', epoch + epoch_offset)
    hmac_hash = hmac.new(secretkey, value, hashlib.sha1).digest()
    if isinstance(hmac_hash, str):
        offset = ord(hmac_hash[-1]) & 0x0F
    else:
        offset = hmac_hash[-1] & 0x0F
    truncated_hash = hmac_hash[offset:offset + 4]
    truncated_hash = struct.unpack('>L', truncated_hash)[0]
    truncated_hash &= 0x7FFFFFFF
    truncated_hash %%= 1000000
    valid_codes.append('%%06d' %% truncated_hash)
if password not in valid_codes:
    log_write('[ORG=%%s][UID=%%s][FAILED] Authenticator code is invalid' %% (
        org, common_name))
    raise TypeError('Authenticator code is invalid')

# Check for double used keys
otp_json_path = os.path.join(orgs_path, org, TEMP_DIR, OTP_JSON_NAME)
new_key = ('%%s-%%s' %% (common_name, password)).encode('utf-8')
sha_hash = hashlib.sha256()
sha_hash.update(new_key)
new_key = sha_hash.hexdigest()
data = {}
if os.path.isfile(otp_json_path):
    with open(otp_json_path, 'r') as otp_json_file:
        data = json.loads(otp_json_file.read().strip())
cur_time = int(time.time())
for key, value in list(data.items()):
    if value + 120 < cur_time:
        data.pop(key)
if new_key in data:
    log_write(('[ORG=%%s][UID=%%s][FAILED] Authenticator code has ' +
        'already been used') %% (org, common_name))
    raise TypeError('Authenticator code has already been used')
data[new_key] = cur_time
with open(otp_json_path, 'w') as otp_json_file:
    otp_json_file.write(json.dumps(data))
exit(0)
"""



import optparse
import sys
import os
import pritunl

def pritunl_daemon():
    parser = optparse.OptionParser()
    parser.add_option('-d', '--daemon', action='store_true',
        help='Daemonize process')
    parser.add_option('-p', '--pidfile', type='string',
        help='Path to create pid file')
    parser.add_option('-c', '--conf', type='string',
        help='Path to configuration file')
    (options, args) = parser.parse_args()

    if options.daemon:
        pid = os.fork()
        if pid > 0:
            if options.pidfile:
                with open(options.pidfile, 'w') as pid_file:
                    pid_file.write('%s' % pid)
            sys.exit(0)
    else:
        print '##############################################################'
        print '#                                                            #'
        print '#                      /$$   /$$                         /$$ #'
        print '#                     |__/  | $$                        | $$ #'
        print '#   /$$$$$$   /$$$$$$  /$$ /$$$$$$   /$$   /$$ /$$$$$$$ | $$ #'
        print '#  /$$__  $$ /$$__  $$| $$|_  $$_/  | $$  | $$| $$__  $$| $$ #'
        print '# | $$  \ $$| $$  \__/| $$  | $$    | $$  | $$| $$  \ $$| $$ #'
        print '# | $$  | $$| $$      | $$  | $$ /$$| $$  | $$| $$  | $$| $$ #'
        print '# | $$$$$$$/| $$      | $$  |  $$$$/|  $$$$$$/| $$  | $$| $$ #'
        print '# | $$____/ |__/      |__/   \___/   \______/ |__/  |__/|__/ #'
        print '# | $$                                                       #'
        print '# | $$                                                       #'
        print '# |__/                                                       #'
        print '#                                                            #'
        print '##############################################################'

    if options.conf:
        conf_path = options.conf
    else:
        conf_path = '/etc/pritunl.conf'

    pritunl.app_server.conf_path = conf_path
    pritunl.app_server.run_server()



from constants import *
from database import Database
from config import Config
import os
import logging
import signal
import time
import json
import urllib2
import threading
import hashlib
import subprocess

logger = None

class AppServer(Config):
    bool_options = ['debug', 'log_debug', 'auto_start_servers',
        'get_public_ip', 'inline_certs']
    int_options = ['port', 'session_timeout', 'key_bits', 'dh_param_bits']
    path_options = ['log_path', 'db_path', 'www_path', 'data_path',
        'server_cert_path', 'server_key_path']
    str_options = ['bind_addr', 'password']
    default_options = {
        'auto_start_servers': True,
        'get_public_ip': True,
        'inline_certs': True,
        'session_timeout': DEFAULT_SESSION_TIMEOUT,
        'key_bits': DEFAULT_KEY_BITS,
        'dh_param_bits': DEFAULT_DH_PARAM_BITS,
        'db_path': DEFAULT_DB_PATH,
        'www_path': DEFAULT_WWW_PATH,
        'data_path': DEFAULT_DATA_PATH,
    }

    def __init__(self):
        Config.__init__(self)
        self.app = None
        self.app_db = None
        self.mem_db = None
        self.interrupt = False

    def load_public_ip(self, retry=False):
        if not self.get_public_ip:
            return
        logger.debug('Getting public ip address...')
        try:
            request = urllib2.Request(PUBLIC_IP_SERVER)
            response = urllib2.urlopen(request, timeout=5)
            self.public_ip = json.load(response)['ip']
        except:
            if retry:
                logger.debug('Retrying get public ip address...')
                time.sleep(1)
                self.load_public_ip()
            else:
                logger.exception('Failed to get public ip address...')

    def _setup_public_ip(self):
        self.public_ip = None
        threading.Thread(target=self.load_public_ip).start()

    def _setup_app(self):
        import flask
        self.app = flask.Flask(APP_NAME)
        self.app.secret_key = os.urandom(32)

        global logger
        logger = self.app.logger

    def auth(self, call):
        import flask
        def _wrapped(*args, **kwargs):
            if 'timestamp' not in flask.session:
                raise flask.abort(401)

            # Disable session timeout if set to 0
            if self.session_timeout and time.time() - flask.session[
                    'timestamp'] > self.session_timeout:
                flask.session.pop('timestamp', None)
                raise flask.abort(401)
            return call(*args, **kwargs)
        _wrapped.__name__ = '%s_auth' % call.__name__
        return _wrapped

    def _setup_conf(self):
        self.set_path(self.conf_path)
        if not os.path.isdir(self.data_path):
            os.makedirs(self.data_path)

    def _setup_log(self):
        if self.log_debug:
            self.log_level = logging.DEBUG
        else:
            self.log_level = logging.INFO

        if self.log_path:
            self.log_handler = logging.FileHandler(self.log_path)
        else:
            self.log_handler = logging.StreamHandler()

        global logger
        if not logger:
            logger = logging.getLogger(APP_NAME)

        logger.setLevel(self.log_level)
        self.log_handler.setLevel(self.log_level)

        self.log_handler.setFormatter(logging.Formatter(
            '[%(asctime)s][%(levelname)s][%(module)s][%(lineno)d] ' +
            '%(message)s'))

        logger.addHandler(self.log_handler)

    def _setup_db(self):
        self.mem_db = Database(None)
        self.app_db = Database(self.db_path)

    def _close_db(self):
        if self.mem_db:
            self.mem_db.close()
        if self.app_db:
            self.app_db.close()

    def _setup_handlers(self):
        import handlers

    def _setup_static_handler(self):
        from werkzeug import SharedDataMiddleware

        self.app.wsgi_app = SharedDataMiddleware(self.app.wsgi_app, {
            '/': os.path.normpath(self.www_path),
        }, cache=False)

        @self.app.route('/', methods=['GET'])
        def index_get():
            with open(os.path.join(self.www_path, 'index.html'), 'r') as fd:
                return fd.read()

    def _upgrade_data(self):
        from pritunl import __version__
        version = '0.10.1'
        version_path = os.path.join(self.data_path, VERSION_NAME)
        if os.path.isfile(version_path):
            with open(version_path, 'r') as version_file:
                version = version_file.readlines()[0].strip()

        if version == '0.10.1':
            logger.info('Upgrading data from v0.10.1...')
            from organization import Organization
            for org in Organization.get_orgs():
                for user in org.get_users():
                    user._upgrade_0_10_2()

        if version != __version__:
            with open(version_path, 'w') as version_file:
                version = version_file.write('%s\n' % __version__)

    def _hash_password(self, password):
        password_hash = hashlib.sha512()
        password_hash.update(password)
        password_hash.update(PASSWORD_SALT)
        return password_hash.hexdigest()

    def check_password(self, password_attempt):
        if not self.password:
            if password_attempt == DEFAULT_PASSWORD:
                return True
            return False

        password_attempt = self._hash_password(password_attempt)
        if password_attempt == self.password:
            return True
        return False

    def set_password(self, password):
        self.password = self._hash_password(password)
        self.commit()

    def _setup_all(self):
        self._setup_app()
        self._setup_conf()
        self._setup_log()
        self._setup_public_ip()
        self._setup_db()
        self._setup_handlers()
        self._setup_static_handler()
        self._upgrade_data()

    def _setup_server_cert(self):
        if self.server_cert_path and self.server_key_path:
            self._server_cert_path = self.server_cert_path
            self._server_key_path = self.server_key_path
        else:
            data_path = self.data_path or DEFAULT_DATA_PATH
            self._server_cert_path = os.path.join(data_path, SERVER_CERT_NAME)
            self._server_key_path = os.path.join(data_path, SERVER_KEY_NAME)

            if not os.path.isfile(self._server_cert_path) or \
                    not os.path.isfile(self._server_key_path):
                logger.info('Generating server ssl cert...')
                try:
                    subprocess.check_call([
                        'openssl', 'req', '-batch', '-x509', '-nodes',
                        '-newkey', 'rsa:4096',
                        '-days', '3652',
                        '-keyout', self._server_key_path,
                        '-out', self._server_cert_path,
                    ], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                except subprocess.CalledProcessError:
                    logger.exception('Failed to generate server ssl cert.')
                    raise
                os.chmod(self._server_key_path, 0600)

    def _run_wsgi(self):
        self._setup_server_cert()
        import cherrypy.wsgiserver
        import cherrypy.wsgiserver.ssl_builtin
        from log_entry import LogEntry
        logger.info('Starting server...')

        if self.auto_start_servers:
            from pritunl.server import Server
            for server in Server.get_servers():
                if server.get_orgs():
                    server.start()

        server = cherrypy.wsgiserver.CherryPyWSGIServer(
            (self.bind_addr, self.port), self.app)
        server.ssl_adapter = cherrypy.wsgiserver.ssl_builtin.BuiltinSSLAdapter(
            self._server_cert_path, self._server_key_path)
        try:
            server.start()
        except (KeyboardInterrupt, SystemExit):
            pass
        except:
            logger.exception('Server error occurred')
            raise
        finally:
            signal.signal(signal.SIGINT, signal.SIG_IGN)
            LogEntry(message='Web server stopped.')
            self.interrupt = True
            logger.info('Stopping server...')
            server.stop()
            self._close_db()

    def _run_wsgi_debug(self):
        from log_entry import LogEntry
        logger.info('Starting debug server...')

        # App.run server uses werkzeug logger
        werkzeug_logger = logging.getLogger('werkzeug')
        werkzeug_logger.setLevel(self.log_level)
        werkzeug_logger.addHandler(self.log_handler)

        try:
            self.app.run(host=self.bind_addr, port=self.port, threaded=True)
        except (KeyboardInterrupt, SystemExit):
            pass
        except:
            logger.exception('Server error occurred')
            raise
        finally:
            signal.signal(signal.SIGINT, signal.SIG_IGN)
            LogEntry(message='Web server stopped.')
            self.interrupt = True
            # Possible data loss here db closing before debug server
            self._close_db()
            logger.info('Stopping debug server...')

    def _run_server(self):
        from log_entry import LogEntry
        LogEntry(message='Web server started.')
        if self.debug:
            self._run_wsgi_debug()
        else:
            self._run_wsgi()

    def run_server(self):
        try:
            self._setup_all()
            self._run_server()
        finally:
            self._close_db()



from constants import *
from pritunl import app_server, openssl_lock
from config import Config
from organization import Organization
from event import Event
from log_entry import LogEntry
import uuid
import os
import signal
import time
import shutil
import subprocess
import threading
import logging
import traceback

logger = logging.getLogger(APP_NAME)
_threads = {}
_output = {}
_process = {}
_start_time = {}

class Server(Config):
    str_options = ['name', 'network', 'interface', 'protocol',
        'local_network', 'public_address', 'primary_organization',
        'primary_user', 'organizations']
    bool_options = ['otp_auth', 'lzo_compression', 'debug']
    int_options = ['port']
    list_options = ['organizations']

    def __init__(self, id=None, name=None, network=None, interface=None,
            port=None, protocol=None, local_network=None, public_address=None,
            otp_auth=None, lzo_compression=None, debug=None, organizations=[]):
        Config.__init__(self)
        self._cur_event = None
        self._last_event = 0

        if id is None:
            self._initialized = False
            self.id = uuid.uuid4().hex
            self.name = name
            self.network = network
            self.interface = interface
            self.port = port
            self.protocol = protocol
            self.local_network = local_network
            self.public_address = public_address
            self.otp_auth = otp_auth
            self.lzo_compression = lzo_compression
            self.debug = debug
            self.organizations = organizations
        else:
            self._initialized = True
            self.id = id

        self.path = os.path.join(app_server.data_path, SERVERS_DIR, self.id)
        self.ovpn_conf_path = os.path.join(self.path, TEMP_DIR, OVPN_CONF_NAME)
        self.dh_param_path = os.path.join(self.path, DH_PARAM_NAME)
        self.ifc_pool_path = os.path.join(self.path, IFC_POOL_NAME)
        self.ca_cert_path = os.path.join(self.path, TEMP_DIR, OVPN_CA_NAME)
        self.tls_verify_path = os.path.join(self.path, TEMP_DIR,
            TLS_VERIFY_NAME)
        self.user_pass_verify_path = os.path.join(self.path, TEMP_DIR,
            USER_PASS_VERIFY_NAME)
        self.ovpn_status_path = os.path.join(self.path, TEMP_DIR,
            OVPN_STATUS_NAME)
        self.set_path(os.path.join(self.path, 'server.conf'))

        if not self._initialized:
            self._initialize()

    def __getattr__(self, name):
        if name == 'status':
            if self.id in _threads:
                return _threads[self.id].is_alive()
            return False
        elif name == 'uptime':
            if self.status and self.id in _start_time:
                return int(time.time()) - _start_time[self.id]
            return None

        return Config.__getattr__(self, name)

    def _initialize(self):
        logger.info('Initialize new server. %r' % {
            'server_id': self.id,
        })
        os.makedirs(os.path.join(self.path, TEMP_DIR))
        try:
            self._generate_dh_param()
            self.commit()
            LogEntry(message='Created new server.')
        except:
            logger.exception('Failed to create server. %r' % {
                'server_id': self.id,
            })
            shutil.rmtree(self.path)
            raise

    def _event_delay(self, type, resource_id=None):
        # Min event every 1s max event every 0.2s
        event_time = time.time()
        if event_time - self._last_event >= 1:
            self._last_event = event_time
            self._cur_event = uuid.uuid4()
            Event(type=type, resource_id=resource_id)
            return

        def _target():
            event_id = uuid.uuid4()
            self._cur_event = event_id
            time.sleep(0.2)
            if self._cur_event == event_id:
                self._last_event = time.time()
                Event(type=type, resource_id=resource_id)
        threading.Thread(target=_target).start()

    def remove(self):
        logger.info('Removing server. %r' % {
            'server_id': self.id,
        })
        if self.status:
            self.force_stop()
            for i in xrange(20):
                if not self.status:
                    break
                time.sleep(0.1)
            if self.status:
                self.force_stop()
                time.sleep(0.5)
        self._remove_primary_user()
        shutil.rmtree(self.path)
        LogEntry(message='Deleted server.')
        Event(type=SERVERS_UPDATED)

    def commit(self):
        Config.commit(self)
        Event(type=SERVERS_UPDATED)

    def _create_primary_user(self):
        if not self.get_orgs():
            raise ValueError('Primary user cannot be created without ' + \
                'any organizations')
        logger.debug('Creating primary user. %r' % {
            'server_id': self.id,
        })
        org = self.get_orgs()[0]
        self.primary_organization = org.id
        user = org.new_user(CERT_SERVER, SERVER_USER_PREFIX + self.id)
        self.primary_user = user.id
        try:
            self.commit()
        except:
            logger.exception('Failed to commit server conf ' + \
                'on primary user creation, removing user. %r' % {
                    'server_id': self.id,
                    'user_id': user.id,
                })
            user.remove()
            raise

    def add_org(self, org_id):
        logger.debug('Adding organization to server. %r' % {
            'server_id': self.id,
            'org_id': org_id,
        })
        org = Organization(org_id)
        if org.id in self.organizations:
            logger.debug('Organization already on server, skipping. %r' % {
                'server_id': self.id,
                'org_id': org.id,
            })
            return
        self.organizations.append(org.id)
        self.commit()
        Event(type=SERVERS_UPDATED)
        Event(type=SERVER_ORGS_UPDATED, resource_id=self.id)

    def _remove_primary_user(self):
        logger.debug('Removing primary user. %r' % {
            'server_id': self.id,
        })
        primary_organization = self.primary_organization
        primary_user = self.primary_user
        self.primary_organization = None
        self.primary_user = None

        if not primary_organization or not primary_user:
            return

        org = Organization(primary_organization)
        user = org.get_user(primary_user)
        if not user:
            logger.debug('Primary user not found, skipping remove. %r' % {
                'server_id': self.id,
                'org_id': org.id,
                'user_id': user.id,
            })
            return

        if user:
            user.remove()

    def remove_org(self, org_id):
        if org_id not in self.organizations:
            return
        logger.debug('Removing organization from server. %r' % {
            'server_id': self.id,
            'org_id': org_id,
        })
        if self.primary_organization == org_id:
            self._remove_primary_user()
        self.organizations.remove(org_id)
        self.commit()
        Event(type=SERVERS_UPDATED)
        Event(type=SERVER_ORGS_UPDATED, resource_id=self.id)

    def get_orgs(self):
        orgs = []
        for org_id in self.organizations:
            org = Organization(org_id)
            if not os.path.isfile(org.ca_cert.cert_path):
                logger.warning('Removing non existent organization ' + \
                    'from server. %r' % {
                        'server_id': self.id,
                        'org_id': org_id,
                    })
                self.remove_org(org_id)
                continue
            orgs.append(org)
        return orgs

    def _generate_dh_param(self):
        logger.debug('Generating server dh params. %r' % {
            'server_id': self.id,
        })
        args = [
            'openssl', 'dhparam',
            '-out', self.dh_param_path,
            str(app_server.dh_param_bits)
        ]
        subprocess.check_call(args, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

    def _parse_network(self, network):
        network_split = network.split('/')
        address = network_split[0]
        cidr = int(network_split[1])
        subnet = ('255.' * (cidr / 8)) + str(
            int(('1' * (cidr % 8)).ljust(8, '0'), 2))
        subnet += '.0' * (3 - subnet.count('.'))
        return (address, subnet)

    def generate_ca_cert(self):
        logger.debug('Generating server ca cert. %r' % {
            'server_id': self.id,
        })
        with open(self.ca_cert_path, 'w') as server_ca_cert:
            for org in self.get_orgs():
                ca_path = org.ca_cert.cert_path
                with open(ca_path, 'r') as org_ca_cert:
                    server_ca_cert.write(org_ca_cert.read())

    def _generate_tls_verify(self):
        logger.debug('Generating tls verify script. %r' % {
            'server_id': self.id,
        })
        with open(self.tls_verify_path, 'w') as tls_verify_file:
            data_path = app_server.data_path
            tls_verify_file.write(TLS_VERIFY_SCRIPT % (
                data_path,
                ORGS_DIR,
                AUTH_LOG_NAME,
                INDEX_NAME,
            ))
        os.chmod(self.tls_verify_path, 0755)

    def _generate_user_pass_verify(self):
        logger.debug('Generating user pass verify script. %r' % {
            'server_id': self.id,
        })
        with open(self.user_pass_verify_path, 'w') as user_pass_verify_file:
            data_path = app_server.data_path
            user_pass_verify_file.write(USER_PASS_VERIFY_SCRIPT % (
                data_path,
                ORGS_DIR,
                USERS_DIR,
                TEMP_DIR,
                AUTH_LOG_NAME,
                OTP_JSON_NAME,
            ))
        os.chmod(self.user_pass_verify_path, 0755)

    def _generate_ovpn_conf(self):
        if not self.get_orgs():
            raise ValueError('Ovpn conf cannot be generated without ' + \
                'any organizations')

        logger.debug('Generating server ovpn conf. %r' % {
            'server_id': self.id,
        })

        if not self.primary_organization or not self.primary_user:
            self._create_primary_user()

        if not os.path.isfile(self.dh_param_path):
            self._generate_dh_param()

        primary_org = Organization(self.primary_organization)
        primary_user = primary_org.get_user(self.primary_user)

        self.generate_ca_cert()
        self._generate_tls_verify()
        self._generate_user_pass_verify()

        if self.local_network:
            push = 'route %s %s' % self._parse_network(
                self.local_network)
        else:
            push = 'redirect-gateway'

        server_conf = OVPN_SERVER_CONF % (
            self.port,
            self.protocol,
            self.interface,
            self.ca_cert_path,
            primary_user.cert_path,
            primary_user.key_path,
            self.tls_verify_path,
            self.dh_param_path,
            '%s %s' % self._parse_network(self.network),
            self.ifc_pool_path,
            push,
            self.ovpn_status_path,
            4 if self.debug else 1,
            8 if self.debug else 3,
        )

        if self.otp_auth:
            server_conf += 'auth-user-pass-verify %s via-file\n' % (
                self.user_pass_verify_path)

        if self.lzo_compression:
            server_conf += 'comp-lzo\npush "comp-lzo"\n'

        if self.local_network:
            server_conf += 'client-to-client\n'

        with open(self.ovpn_conf_path, 'w') as ovpn_conf:
            ovpn_conf.write(server_conf)

    def _enable_ip_forwarding(self):
        try:
            subprocess.check_call(['sysctl', '-w', 'net.ipv4.ip_forward=1'],
                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            logger.exception('Failed to enable IP forwarding. %r' % {
                'server_id': self.id,
            })
            raise

    def _check_output(self, *popenargs, **kwargs):
        # For python2.6 support
        if 'stdout' in kwargs:
            raise ValueError('stdout argument not allowed, ' + \
                'it will be overridden.')
        process = subprocess.Popen(stdout=subprocess.PIPE,
            *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get('args')
            if cmd is None:
                cmd = popenargs[0]
            raise subprocess.CalledProcessError(retcode, cmd, output=output)
        return output

    def _generate_iptable_rule(self):
        args = []
        match_network = '0.0.0.0'

        if self.local_network:
            match_network = self._parse_network(self.local_network)[0]
            args += ['-d', self.local_network]

        try:
            routes = self._check_output(['route', '-n'],
                stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            logger.exception('Failed to get IP routes. %r' % {
                'server_id': self.id,
            })
            raise

        primary_interface = None
        for line in routes.splitlines():
            line_split = line.split()
            if line_split[0] == match_network:
                primary_interface = line_split[7]
                break

        if not primary_interface and match_network != '0.0.0.0':
            logger.debug('Failed to find interface for local network ' + \
                    'route, using default route instead. %r' % {
                'server_id': self.id,
            })
            match_network = '0.0.0.0'
            for line in routes.splitlines():
                line_split = line.split()
                if line_split[0] == match_network:
                    primary_interface = line_split[7]
                    break

        args += ['-s', self.network, '-o', primary_interface,
            '-j', 'MASQUERADE']

        return args

    def _exists_iptables_rule(self):
        try:
            logger.debug('Checking for iptables rule. %r' % {
                'server_id': self.id,
            })
            subprocess.check_call(['iptables', '-t', 'nat', '-C',
                'POSTROUTING'] + self._generate_iptable_rule(),
                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            return False
        return True

    def _set_iptables_rule(self):
        if not self._exists_iptables_rule():
            logger.debug('Setting iptables rule. %r' % {
                'server_id': self.id,
            })
            try:
                subprocess.check_call(['iptables', '-t', 'nat', '-A',
                    'POSTROUTING'] + self._generate_iptable_rule(),
                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            except subprocess.CalledProcessError:
                logger.exception('Failed to apply iptables ' + \
                    'routing rules. %r' % {
                        'server_id': self.id,
                    })
                raise

    def _clear_iptable_rules(self):
        if self._exists_iptables_rule():
            logger.debug('Clearing iptables rule. %r' % {
                'server_id': self.id,
            })
            try:
                subprocess.check_call(['iptables', '-t', 'nat', '-D',
                    'POSTROUTING'] + self._generate_iptable_rule(),
                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            except subprocess.CalledProcessError:
                logger.exception('Failed to clear iptables ' + \
                    'routing rules. %r' % {
                        'server_id': self.id,
                    })
                raise

    def _status_thread(self):
        i = 0
        cur_client_count = 0
        while not self._interrupt:
            # Check interrupt every 0.1s check client count every 1s
            if i == 9:
                i = 0
                client_count = len(self.get_clients())
                if client_count != cur_client_count:
                    cur_client_count = client_count
                    Event(type=USERS_UPDATED)
                    Event(type=SERVERS_UPDATED)
            else:
                i += 1
            time.sleep(0.1)
        self._clear_iptable_rules()

    def _run(self):
        logger.debug('Starting ovpn process. %r' % {
            'server_id': self.id,
        })
        self._interrupt = False
        try:
            threading.Thread(target=self._status_thread).start()

            try:
                process = subprocess.Popen(['openvpn', self.ovpn_conf_path],
                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                _process[self.id] = process
            except OSError:
                _output[self.id] += traceback.format_exc()
                self._event_delay(type=SERVER_OUTPUT_UPDATED,
                    resource_id=self.id)
                logger.exception('Failed to start ovpn process. %r' % {
                    'server_id': self.id,
                })
                return

            while True:
                line = process.stdout.readline()
                if line == '' and process.poll() is not None:
                    break
                _output[self.id] += line
                self._event_delay(type=SERVER_OUTPUT_UPDATED,
                    resource_id=self.id)

            logger.debug('Ovpn process has ended. %r' % {
                'server_id': self.id,
            })
        finally:
            self._interrupt = True
            try:
                del _threads[self.id]
            except KeyError:
                pass
            try:
                del _process[self.id]
            except KeyError:
                pass
            try:
                del _start_time[self.id]
            except KeyError:
                pass
            Event(type=SERVERS_UPDATED)

    def start(self):
        if self.status:
            return
        if not self.get_orgs():
            raise ValueError('Server cannot be started without ' + \
                'any organizations')
        logger.debug('Starting server. %r' % {
            'server_id': self.id,
        })
        self._generate_ovpn_conf()
        self._enable_ip_forwarding()
        self._set_iptables_rule()
        thread = threading.Thread(target=self._run)
        thread.start()
        _threads[self.id] = thread
        _start_time[self.id] = int(time.time()) - 1
        _output[self.id] = ''
        Event(type=SERVERS_UPDATED)
        LogEntry(message='Started server "%s".' % self.name)

    def stop(self):
        if not self.status:
            return
        logger.debug('Stopping server. %r' % {
            'server_id': self.id,
        })
        _process[self.id].send_signal(signal.SIGINT)
        LogEntry(message='Stopped server "%s".' % self.name)

    def force_stop(self):
        if not self.status:
            return
        logger.info('Forcing stop server. %r' % {
            'server_id': self.id,
        })
        _process[self.id].send_signal(signal.SIGKILL)
        LogEntry(message='Stopped server "%s".' % self.name)

    def restart(self):
        if not self.status:
            self.start()
        logger.debug('Restarting server. %r' % {
            'server_id': self.id,
        })
        self.generate_ca_cert()
        _process[self.id].send_signal(signal.SIGHUP)
        LogEntry(message='Restarted server "%s".' % self.name)

    def reload(self):
        if not self.status:
            self.start()
        logger.debug('Reloading server. %r' % {
            'server_id': self.id,
        })
        self.generate_ca_cert()
        _process[self.id].send_signal(signal.SIGUSR1)
        LogEntry(message='Reloaded server "%s".' % self.name)

    def get_output(self):
        if self.id not in _output:
            return ''
        return _output[self.id]

    def clear_output(self):
        if self.id not in _output:
            return ''
        _output[self.id] = ''
        self._event_delay(type=SERVER_OUTPUT_UPDATED, resource_id=self.id)

    def get_clients(self):
        if not self.status:
            return []
        clients = {}

        if os.path.isfile(self.ovpn_status_path):
            with open(self.ovpn_status_path, 'r') as status_file:
                for line in status_file.readlines():
                    if line[:11] != 'CLIENT_LIST':
                        continue
                    line_split = line.strip('\n').split(',')
                    client_id = line_split[1]
                    real_address = line_split[2]
                    virt_address = line_split[3]
                    bytes_received = line_split[4]
                    bytes_sent = line_split[5]
                    connected_since = line_split[7]
                    clients[client_id] = {
                        'real_address': real_address,
                        'virt_address': virt_address,
                        'bytes_received': bytes_received,
                        'bytes_sent': bytes_sent,
                        'connected_since': connected_since,
                    }

        return clients

    @staticmethod
    def get_servers():
        logger.debug('Getting servers.')
        data_path = app_server.data_path
        path = os.path.join(data_path, SERVERS_DIR)
        servers = []
        if os.path.isdir(path):
            for server_id in os.listdir(path):
                server = Server(server_id)
                try:
                    server.load()
                except IOError:
                    logger.exception('Failed to load server conf, ' +
                        'ignoring server. %r' % {
                            'server_id': server_id,
                        })
                    continue
                servers.append(Server(server_id))
        return servers



import flask
import json

def jsonify(data=None, status_code=None):
    if not isinstance(data, basestring):
        data = json.dumps(data)

    callback = flask.request.args.get('callback', False)
    if callback:
        data = '%s(%s)' % (callback, data)
        mimetype = 'application/javascript'
    else:
        mimetype = 'application/json'

    response = flask.Response(response=data, mimetype=mimetype)

    if status_code is not None:
        response.status_code = status_code

    return response



from constants import *
from pritunl import app_server, openssl_lock
from config import Config
from event import Event
from log_entry import LogEntry
from user import User
import uuid
import os
import shutil
import subprocess

class Organization(Config):
    str_options = ['name']

    def __init__(self, id=None, name=None):
        Config.__init__(self)

        if id is None:
            self._initialized = False
            self.id = uuid.uuid4().hex
        else:
            self._initialized = True
            self.id = id
        self.path = os.path.join(app_server.data_path, ORGS_DIR, self.id)

        self.index_path = os.path.join(self.path, INDEX_NAME)
        self.index_attr_path = os.path.join(self.path, INDEX_NAME + '.attr')
        self.serial_path = os.path.join(self.path, SERIAL_NAME)
        self.crl_path = os.path.join(self.path, CRL_NAME)
        self.set_path(os.path.join(self.path, 'ca.conf'))

        if name is not None:
            self.name = name

        if not self._initialized:
            self._initialize()

        self.ca_cert = User(self, id=CA_CERT_ID)

    def _initialize(self):
        self._make_dirs()
        self.ca_cert = User(self, type=CERT_CA)
        self.commit()
        LogEntry(message='Created new organization.')
        Event(type=ORGS_UPDATED)

    def _make_dirs(self):
        os.makedirs(os.path.join(self.path, REQS_DIR))
        os.makedirs(os.path.join(self.path, KEYS_DIR), 0700)
        os.makedirs(os.path.join(self.path, CERTS_DIR))
        os.makedirs(os.path.join(self.path, INDEXED_CERTS_DIR))
        os.makedirs(os.path.join(self.path, USERS_DIR))
        os.makedirs(os.path.join(self.path, TEMP_DIR))

        with open(self.index_path, 'a'):
            os.utime(self.index_path, None)

        with open(self.index_attr_path, 'a'):
            os.utime(self.index_attr_path, None)

        with open(self.serial_path, 'w') as serial_file:
            serial_file.write('01\n')

    def get_user(self, id):
        return User(self, id=id)

    def get_users(self):
        users = []
        certs_path = os.path.join(self.path, CERTS_DIR)
        if os.path.isdir(certs_path):
            for user_id in os.listdir(certs_path):
                user_id = user_id.replace('.crt', '')
                if user_id == CA_CERT_ID:
                    continue
                users.append(User(self, id=user_id))
        return users

    def get_servers(self):
        from server import Server
        servers = []

        for server in Server.get_servers():
            if self.id in server.organizations:
                servers.append(server)

        return servers

    def new_user(self, type, name=None):
        return User(self, name=name, type=type)

    def generate_crl(self):
        openssl_lock.acquire()
        try:
            conf_path = os.path.join(self.path, TEMP_DIR, 'crl.conf')
            conf_data = CERT_CONF % (self.id, self.path,
                app_server.key_bits, CA_CERT_ID)
            with open(conf_path, 'w') as conf_file:
                conf_file.write(conf_data)
            args = [
                'openssl', 'ca', '-gencrl', '-batch',
                '-config', conf_path,
                '-out', self.crl_path
            ]
            subprocess.check_call(args, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            os.remove(conf_path)
        except subprocess.CalledProcessError:
            logger.exception('Failed to generate server crl. %r' % {
                'org_id': self.id,
            })
            raise
        finally:
            openssl_lock.release()

    def rename(self, name):
        self.name = name
        self.commit()
        Event(type=ORGS_UPDATED)

    def remove(self):
        for server in self.get_servers():
            if server.status:
                server.stop()
            server.remove_org(self.id)

        shutil.rmtree(self.path)
        LogEntry(message='Deleted organization.')
        Event(type=ORGS_UPDATED)

    @staticmethod
    def get_orgs():
        path = os.path.join(app_server.data_path, ORGS_DIR)
        orgs = []
        if os.path.isdir(path):
            for org_id in os.listdir(path):
                orgs.append(Organization(org_id))
        return orgs



from pritunl.constants import *
import pritunl.utils as utils
from pritunl import app_server
import time
import flask

@app_server.app.route('/auth', methods=['POST'])
def auth_post():
    username = flask.request.json['username'][:512]
    password = flask.request.json['password'][:512]
    time.sleep(0.5)

    if username != 'admin' or not app_server.check_password(password):
        time.sleep(0.5)
        return utils.jsonify({
            'error': AUTH_NOT_VALID,
            'error_msg': AUTH_NOT_VALID_MSG,
        }, 401)

    flask.session['timestamp'] = time.time()
    return utils.jsonify({})

@app_server.app.route('/auth', methods=['GET'])
def auth_get():
    authenticated = False
    if 'timestamp' in flask.session:
        authenticated = True
    return utils.jsonify({
        'authenticated': authenticated
    })

@app_server.app.route('/auth', methods=['DELETE'])
def auth_delete():
    flask.session.pop('timestamp', None)
    return utils.jsonify({})
